module DocMachine::Container

  class ContainerError < Exception ; end

  class SaveError < ContainerError ; end
  class KillError < ContainerError ; end
  class RemoveError < ContainerError ; end
  class LoadError < ContainerError ; end
  class RunError < ContainerError ; end
  class PullError < ContainerError ; end

  abstract class AbstractContainerEngine
    # Pulls the specified Docker/Podman image
    abstract def pull_image(image_tag : String) : Nil

    # Loads the Docker/Podman image from a local cache
    abstract def load_image(cache_path : String) : Nil

    # Saves the Docker/Podman image to a local cache
    abstract def save_image(image_tag : String, cache_path : String) : Nil

    # Runs a container with the given options
    abstract def run_container(
      image_tag : String,
      action : String,
      docker_name : String,
      docker_opts : Array(String),
      enable_tty : Bool
    ) : Process

    # Kills a running container by name
    abstract def kill_container(container_name : String) : Nil

    # Removes a container by name
    abstract def remove_container(container_name : String) : Nil

    # Finds the container ID based on its name
    abstract def find_container_id(name : String) : String
  end

end
